@@ -5,7 +5,7 @@
 # =============================================================================
 # Standard data analysis
 import pandas as pd
-from numpy.random import random as randn
+from numpy.random import random
 import time
 from statsmodels.api import tsa
 
@@ -25,9 +25,6 @@ class Trendex:
     geo: str
         The country or place the search is conducted in, see Trends documentation.
 
-    lang: str, options are currently 'en' for english and 'es' for spanish
-        The language that the kw list is (required for optimal benchmark selection).
-
     date_start: str, optional
         The data where the index starts in format: 'yyyy-mm-dd',
         if none provided, then does it at the cutoff days before the date end.
@@ -67,43 +64,32 @@ class Trendex:
         adjusted trends are always constructed and saved in trends_sa,
         but this incorporates them into the index that is automatically constructed.
 
-    sa_method: str, optional (default = 'trends')
-        Three methods are acceptable: 'trends' or 'index' or 'both'.
-        The trends method seasonally adjusts each trend separately (before adding).
-        The index method seasonally adjusts only the final series.
-        The both method seasonally adjusts separately and the final index.
-
-
-
-
     Returns (back to class instance)
     -------
-    self.gti: Series
+    self.gti: Series (main output)
         This is the normalized indexes made from the underlying data. It is
         the main thing returned from this function.
 
-    self.trends: Dataframe
+    self.trends: Dataframe (major output)
         It is the adjusted and combined series for each term searched.
         You could use this to plot individual keywords in the index.
         Will differ from raw_trends_adjusted if frequency is changed from daily.
 
-    self.trends_sa: Dataframe
+    self.trends_sa: Dataframe (major output)
         It is the trends dataframe where each trend has been seasonally adjusted.
 
-    self.raw_trends_adjusted: Dictionary
+    self.raw_trends_adjusted: Dictionary (minor output)
         These are the adjusted (using overlapping timeframes) raw results
         for each term. Index of dictionary corresponds to index of timechunks.
         Could differ from trends if frequency is changed from daily.
 
-    self.raw_trends: Dictionary
+    self.raw_trends: Dictionary (minor output)
         These are the unadjusted raw results for each term.
         Note: Adjustment has still been made by the benchmark term for
         searches exceeding 5 terms.
 
-    self.adjustment_factors: Series
+    self.adjustment_factors: Series (minor output)
         Returns the adjustment factors used on each overlapping segment.
-        [term]_1 is the mean of the term in the earlier segment.
-        [term]_2 is the mean of the term in the later segment.
         The adjustment is [term]_1/[term_2] * segment_2
         Note: means for each segment are bounded from below by 1, so that
         we do not seriously alter indices.
@@ -113,23 +99,21 @@ class Trendex:
     pytrend = TrendReq(tz=300) # tz is the timezone offset, in this case EST
 
     # Universal parameter(s)
-    cutoff = 270 - 20 # google returns max 270 values; I make it 250 just in case.
-    overlap = 30 # An arbitrary length of time for the series to overlap
+    cutoff_d = 270 - 10 # google returns max 270 values; I make it 260 just in case.
+    cutoff_m = 270*7 + 10
+    overlap = 45 # An arbitrary (long) length of time for the series to overlap
     kw_limit = 20 # Default is to break kw_list into chunks with "+" operator
 
-    def __init__(self, kw_list, geo, lang, date_start=None, date_end=None,
-                 frequency='daily', gen_index=True, plot=True,
-                 kw_list_split=True, benchmark_select=True, slowdown=True,
-                 seasonal_adjust=True, sa_method='trends'):
+    def __init__(self, kw_list, geo, date_start=None, date_end=None,
+                 frequency='daily', gen_index=True, plot=True, seasonal_adjust=True,
+                 kw_list_split=True, benchmark_select=True, slowdown=True):
 
         # Input Arguments
         self.user_kw_list = kw_list.copy()
         self.geo = geo
-        self.lang = lang
         self.user_date_start = date_start
         self.user_date_end = date_end
         self.frequency = frequency
-        self.sa_method = sa_method
         self.seasonal = seasonal_adjust
         self.slowdown = slowdown
         self.benchmark_select = benchmark_select
@@ -142,7 +126,10 @@ class Trendex:
         self.date_start, self.date_end = self.auto_dates()
         self.benchmark, self.search_groups = self.get_benchmark()
 
-        self.timechunks = self.get_timechunks()
+        if frequency == 'daily' or frequency == 'weekly':
+            self.timechunks = self.get_timechunks()
+        else:
+            self.timechunks = [[self.date_start,self.date_end]]
 
         # Initialized None Arguments For make_index
         self.raw_trends = None
@@ -215,34 +202,35 @@ class Trendex:
         self.adjustment_factors = {}
         # Loop through and get all the separate time frames (timechunks makes the intervals)
         for ii, dd in enumerate(self.timechunks):
-            if self.slowdown:
-                time.sleep(round(randn()*6+2,2)) # sleep it so no timeout
             # iterate through and pull the timeframes
             temp_trends = self.pull_timeframe(date_start=dd[0],date_end=dd[1])
-            self.raw_trends[ii] = temp_trends
+            self.raw_trends[ii] = temp_trends.copy()
             if ii==0:
-                trends = temp_trends
+                trends = temp_trends.copy()
             else:
+                temp_trends = temp_trends.copy()
+                if self.slowdown:
+                    time.sleep(round(random()*6+2,2)) # sleep it so no timeout
                 # Calculate the means of the overlapping part
-                meanadj = trends.join(temp_trends,how='inner',
-                                      lsuffix='_1',rsuffix='_2').mean()
-
-                # make it so that the minimum is 1 when we do this overlap
-                meanadj = meanadj.apply(lambda x: 1 if x<1 else x)
+                meanadj = trends.join(temp_trends.copy(),how='inner',
+                                      lsuffix='_1',rsuffix='_2').replace({0:1})
+                for jj in temp_trends.columns:
+                    meanadj['%s' %jj] = meanadj['%s_1' %jj]/meanadj['%s_2' %jj]
+                    meanadj = meanadj.drop(columns=['%s_1' %jj,'%s_2' %jj])
 
+                meanadj = meanadj.mean()
                 # save them too so we can recover them if needed
-                self.adjustment_factors[ii] = meanadj
+                self.adjustment_factors[ii] = meanadj.copy()
 
                 for jj in temp_trends.columns:
                     # adjust the following parts to have the same overlap mean
-                    temp_trends.loc[:,jj] = temp_trends.loc[:,jj].values*\
-                        (meanadj['%s_1' %jj]/meanadj['%s_2' %jj])
+                    temp_trends.loc[:,jj] = temp_trends.loc[:,jj].values*meanadj[jj]
 
                 # append the new part
-                trends = trends.append(temp_trends.iloc[self.overlap+1:])
+                trends = trends.append(temp_trends.iloc[self.overlap+1:]).copy()
 
         # Save the adjusted trends too
-        self.raw_trends_adjusted = trends
+        self.raw_trends_adjusted = trends.copy()
 
         if self.frequency == 'daily':
             pass
@@ -253,47 +241,30 @@ class Trendex:
             trends = trends.loc[trends.timecount.eq(7),:] # make sure no partial weeks
             trends = trends.groupby('date')[self.kw_list].mean()
         elif self.frequency == 'monthly':
-            trends = trends.reset_index()
-            trends.date = trends.date.dt.to_period('M').dt.to_timestamp() # make monthly
-            trends.loc[:,'timecount'] = trends.groupby('date')['date'].transform('count')
-            trends = trends.loc[trends.timecount.eq(trends.date.dt.days_in_month),:]
-            trends = trends.groupby('date')[self.kw_list].mean()
+            pass
         elif self.frequency == 'quarterly':
             trends = trends.reset_index()
             trends.date = trends.date.dt.to_period('Q').dt.to_timestamp() # make qtrly
             trends.loc[:,'timecount'] = trends.groupby('date')['date'].transform('count')
-            trends = trends.loc[trends.timecount.ge(28*3),:] # minimum February 3 times
+            trends = trends.loc[trends.timecount.eq(3),:] # has to have 3 months
             trends = trends.groupby('date')[self.kw_list].mean()
 
         # Save also the collapsed trend series
-        self.trends = trends
+        self.trends = trends.copy()
         trends_sa = trends.apply(self.sadjust,axis=0)
-        self.trends_sa = trends_sa
+        self.trends_sa = trends_sa.copy()
 
-
-        # Old way
-        # indices = pd.concat([
-        #         trends_sa.apply(self.normalize).sum(axis=1).to_frame('GTI (Normalized)'),
-        #         trends_sa.sum(axis=1).to_frame('GTI (Standard)')
-        #         ],axis=1).apply(self.normalize)
-        if self.seasonal and self.sa_method == 'trends':
-            indices = self.normalize(trends_sa.sum(axis=1))
-            indices.name = 'GTI'
-        elif self.seasonal and self.sa_method == 'index':
+        if self.seasonal:
             indices = self.normalize(self.sadjust(trends.sum(axis=1)))
-            indices.name = 'GTI'
-        elif self.seasonal and self.sa_method == 'both':
-            indices = self.normalize(self.sadjust(trends_sa.sum(axis=1)))
-            indices.name = 'GTI'
         else:
             indices = self.normalize(trends.sum(axis=1))
-            indices.name = 'GTI'
+        indices.name = 'GTI'
 
         if plot:
             indices.plot()
 
         # The final thing is to add the indices
-        self.gti = indices
+        self.gti = indices.copy()
 
         return self
 
@@ -340,18 +311,20 @@ class Trendex:
         if not date_end:
             date_end = self.date_end
 
+        small_dum = False
 
         if self.benchmark:
             # Do the searches in batches
             # For each one initial transform and spit into dictionary of dataframes
             for idx, ss in enumerate(self.search_groups):
                 if self.slowdown:
-                    time.sleep(round(randn()*4+2,2)) # sleep it so no timeout
+                    time.sleep(round(random()*4+2,2)) # sleep it so no timeout
                 # Do the search
                 self.pytrend.build_payload(ss,geo=self.geo,
                                       timeframe='%s %s' %(date_start,
                                                           date_end))
                 df = self.pytrend.interest_over_time()
+                df = df.copy()
 
                 # Warn and stop if benchmark sucks
                 if not self.benchmark_select:
@@ -362,8 +335,7 @@ class Trendex:
                         break
                 else:
                     if self.too_small(df[self.benchmark]):
-                        print('Benchmark term %s is optimally selected, but performs '\
-                            'poorly between %s and %s' %(self.benchmark,date_start,date_end))
+                        small_dum = True
 
                 # Get rid of partial days
                 df = df.loc[df.isPartial.astype('str').eq('False'),ss]
@@ -372,18 +344,26 @@ class Trendex:
                 df[self.benchmark] = df[self.benchmark].replace({0:1})
 
                 if idx==0:
-                    trends = df
+                    frame = df.copy()
                 else:
-                    df = df.mul((trends[self.benchmark]/df[self.benchmark]).values,0)
-                    trends = trends.join(df.drop(self.benchmark,axis=1))
+                    adjframe = frame.join(df,how='inner',
+                                          lsuffix='_1',rsuffix='_2').copy()
+                    factor = (adjframe[self.benchmark+'_1'].values/
+                                  adjframe[self.benchmark+'_2'].values).mean()
+                    df = df*factor
+                    frame = frame.join(df.drop(self.benchmark,axis=1).copy()).copy()
 
         else:
             self.pytrend.build_payload(self.kw_list,geo=self.geo,
                                   timeframe='%s %s' %(date_start,date_end))
             df = self.pytrend.interest_over_time()
-            trends = df.loc[df.isPartial.astype('str').eq('False'),self.kw_list]
+            frame = df.loc[df.isPartial.astype('str').eq('False'),self.kw_list]
 
-        return trends
+        if small_dum:
+            print('Benchmark term %s is optimal, but performs '\
+                  'poorly between %s and %s' %(self.benchmark,date_start,date_end))
+
+        return frame
 
 
     def get_benchmark(self):
@@ -408,11 +388,26 @@ class Trendex:
         else:
             date_end = self.user_date_end
 
-        if not self.user_date_start:
-            date_start = (pd.to_datetime(date_end) -
-                          pd.Timedelta(days=self.cutoff)).strftime('%Y-%m-%d')
-        else:
-            date_start = self.user_date_start
+        # then deal with daily/weekly or monthly/quarterly
+        if self.frequency == 'daily' or self.frequency == 'weekly':
+            if not self.user_date_start:
+                date_start = (pd.to_datetime(date_end) -
+                              pd.Timedelta(days=self.cutoff_d)).strftime('%Y-%m-%d')
+            else:
+                date_start = self.user_date_start
+
+        elif self.frequency == 'monthly' or self.frequency == 'quarterly':
+            if not self.user_date_start:
+                date_start = (pd.to_datetime(date_end) -
+                              pd.Timedelta(days=self.cutoff_m)).strftime('%Y-%m-%d')
+            else:
+                datelength = (pd.to_datetime(date_end) -
+                              pd.to_datetime(self.user_date_start)).days
+                if datelength<2000:
+                    date_start = (pd.to_datetime(date_end) -
+                                  pd.Timedelta(days=2000)).strftime('%Y-%m-%d')
+                else:
+                    date_start = self.user_date_start
 
         return date_start, date_end
 
@@ -421,15 +416,15 @@ class Trendex:
         # if ending date was left blank, assume we want until today
         daterange = (pd.to_datetime(self.date_end) - pd.to_datetime(self.date_start)).days
 
-        if daterange>self.cutoff:
+        if daterange>self.cutoff_d:
             # see what length of time we are working with total
             # split it up into the overlapping chunks
             lst = [list(a) for a in zip(range(0,
                                               daterange+1,
-                                              self.cutoff-self.overlap),
-                                        range(self.cutoff,
-                                              daterange+self.cutoff,
-                                              self.cutoff-self.overlap))]
+                                              self.cutoff_d-self.overlap),
+                                        range(self.cutoff_d,
+                                              daterange+self.cutoff_d,
+                                              self.cutoff_d-self.overlap))]
             # make sure that the last value lines up with our actual last value
             lst[-1][1] = daterange
 
@@ -448,20 +443,15 @@ class Trendex:
         """
         Run generic searches over the timeframe to calculate best potential index:
 
-        The function takes the self object, analyzes the language of the kw_list and
-        runs searches over the entire timeframe, comparing the average of the series to
-        football (english) or futbol + fútbol (spanish).
+        The function takes the self object, and runs searches over the entire timeframe,
+        comparing the average of the series to football or variants of that word,
+        since this is a highly searched universal word.
 
         The highest average over the timeframe (making sure there are not years
         with 0 average is selected as benchmark).
         """
 
-        if self.lang == 'es':
-            popterm = 'fútbol'
-        elif self.lang == 'en':
-            popterm = 'football'
-        else:
-            raise ValueError('Currently only supports English (en) and Spanish (es)')
+        popterm = 'football + fútbol + futbol + futebol + Fußball + calcio'
 
         chunks = list(self.chunks([popterm]+self.kw_list))
 
